/*
 *      CAPS: Computational Aircraft Prototype Syntheses
 *
 *             Geometry Input test
 *
 *      Copyright 2014-2015, Massachusetts Institute of Technology
 *      Licensed under The GNU Lesser General Public License, version 2.1
 *      See http://www.opensource.org/licenses/lgpl-2.1.php
 *
 */

#include "caps.h"

extern void printObjects( capsObj object, int indent );


int main(int argc, char *argv[])
{
  int            i, j, stat, nGeomIn, index, dim, nrow, ncol, nErr, nLines;
  double         val;
  char           **lines;
  enum capsFixed lfixed, sfixed;
  enum capsNull  nval;
  capsObj        pobj, obj;
  capsErrs       *errors;

  if ((argc < 2) || (argc > 3)) {
    printf(" usage: geomIn filename [capsname]!\n");
    return 1;
  }

  stat = caps_open(argv[1], "geomIn", &pobj);
  if (stat != CAPS_SUCCESS) {
    printf(" caps_open = %d\n", stat);
    return 1;
  }
  
  stat = caps_size(pobj, VALUE, GEOMETRYIN, &nGeomIn);
  if (stat != CAPS_SUCCESS) {
    printf(" CAPS Error: returns %d from caps_size(GeomIn)!\n", stat);
    stat = caps_close(pobj);
    if (stat != CAPS_SUCCESS) printf(" caps_close = %d\n", stat);
    return 1;
  }
  if (nGeomIn == 0) {
    printf("\n No Geometry Inouts to modify!\n\n");
    stat = caps_close(pobj);
    if (stat != CAPS_SUCCESS) printf(" caps_close = %d\n", stat);
    return 1;
  }
  printf("\n");
  for (i = 0; i < nGeomIn; i++) {
    stat = caps_childByIndex(pobj, VALUE, GEOMETRYIN, i+1, &obj);
    if (stat != CAPS_SUCCESS) {
      printf(" CAPS Error: GeomIn[%d] ret=%d from caps_childbyIndex!\n",
             i+1, stat);
      stat = caps_close(pobj);
      if (stat != CAPS_SUCCESS) printf(" caps_close = %d\n", stat);
      return 1;
    }
    printf(" %d:", i+1);
    printObjects(obj, 0);
  }
  printf("\n");
  
  do {
    printf(" Enter Index [0=done] & value: ");
    scanf("%d %lf", &index, &val);
    if (index <= 0) break;
    if (index > nGeomIn) {
      printf("\n Error: Index Out of Range [1-%d]!\n", nGeomIn);
      continue;
    }
    stat = caps_childByIndex(pobj, VALUE, GEOMETRYIN, index, &obj);
    if (stat != CAPS_SUCCESS) {
      printf(" CAPS Error: GeomIn[%d] ret=%d from caps_childbyIndex!\n",
             index, stat);
      stat = caps_close(pobj);
      if (stat != CAPS_SUCCESS) printf(" caps_close = %d\n", stat);
      return 1;
    }
    stat = caps_getValueShape(obj, &dim, &lfixed, &sfixed, &nval,
                              &nrow, &ncol);
    if (stat != CAPS_SUCCESS) {
      printf(" CAPS Error: GeomIn[%d] ret=%d from caps_getValueShape!\n",
             index, stat);
      stat = caps_close(pobj);
      if (stat != CAPS_SUCCESS) printf(" caps_close = %d\n", stat);
      return 1;
    }
    if (dim != 0) {
      printf("\n Error: Dimension must be 0 and is %d!\n", dim);
      continue;
    }
    stat = caps_setValue(obj, 1, (void *) &val);
    if (stat != CAPS_SUCCESS) {
      printf(" CAPS Error: GeomIn[%d] ret=%d from caps_setValue!\n",
             index, stat);
      stat = caps_close(pobj);
      if (stat != CAPS_SUCCESS) printf(" caps_close = %d\n", stat);
      return 1;
    }
  } while (index > 0);
  
  /* special mode to do a geometry (only) regen */
  stat = caps_preAnalysis(pobj, &nErr, &errors);
  if (stat != CAPS_SUCCESS) {
    if (errors != NULL) {
      for (i = 0; i < nErr; i++) {
        stat = caps_errorInfo(errors, i+1, &obj, &nLines, &lines);
        if (stat != CAPS_SUCCESS) {
          printf(" CAPS Error: caps_errorInfo[%d] = %d!\n", i+1, stat);
          continue;
        }
        for (j = 0; j < nLines; j++) printf(" %s\n", lines[j]);
      }
      caps_freeError(errors);
    }
    printf(" CAPS Error: regen ret=%d from caps_preAnalysis nErr = %d!\n",
           stat, nErr);
    stat = caps_close(pobj);
    if (stat != CAPS_SUCCESS) printf(" caps_close = %d\n", stat);
    return 1;
  }
  
  printf("\n");
  printObjects(pobj, 0);
  printf("\n");
  
  if (argc == 3) {
    stat = caps_save(pobj, argv[2]);
    if (stat != CAPS_SUCCESS) printf(" caps_save = %d\n", stat);
  }
  
  stat = caps_close(pobj);
  if (stat != CAPS_SUCCESS) printf(" caps_close = %d\n", stat);

  return 0;
}
